@@ -382,8 +382,8 @@ nsJPEGDecoder::ReadJPEGData(const char* aData, size_t aLength)
     jpeg_calc_output_dimensions(&mInfo);
 
     MOZ_ASSERT(!mImageData, "Already have a buffer allocated?");
-    nsresult rv = AllocateFrame(/* aFrameNum = */ 0, OutputSize(),
-                                FullOutputFrame(), SurfaceFormat::B8G8R8X8);
+    nsresult rv = AllocateFrame(OutputSize(), FullOutputFrame(),
+                                SurfaceFormat::B8G8R8X8);
     if (NS_FAILED(rv)) {
       mState = JPEG_ERROR;
       MOZ_LOG(sJPEGDecoderAccountingLog, LogLevel::Debug,
