@@ -21,6 +21,10 @@
 #include "nsServiceManagerUtils.h"
 #include "nsNetCID.h"
 
+#if defined(XP_MACOSX)
+#  include "nsMacUtilsImpl.h"
+#endif
+
 namespace mozilla {
 namespace net {
 
@@ -725,6 +729,10 @@ ProxyAutoConfig::SetupJS()
   mJSNeedsSetup = false;
   MOZ_ASSERT(!GetRunning(), "JIT is running");
 
+#if defined(XP_MACOSX)
+  nsMacUtilsImpl::EnableTCSMIfAvailable();
+#endif
+
   delete mJSContext;
   mJSContext = nullptr;
 
